/*------------------------------------------------------------------------------
* rtklib unit test driver : rinex function
*-----------------------------------------------------------------------------*/
#include <stdio.h>
#include <assert.h>
#include "../../src/rtklib.h"

static void dumpobs(obs_t *obs)
{
	int i;
	char str[64];
	printf("obs : n=%d\n",obs->n);
	for (i=0;i<obs->n;i++) {
		time2str(obs->data[i].time,str,3);
		printf("%s : %2d %2d %13.3f %13.3f %13.3f %13.3f  %d %d\n",str,obs->data[i].sat,
			   obs->data[i].rcv,obs->data[i].L[0],obs->data[i].L[1],
			   obs->data[i].P[0],obs->data[i].P[1],obs->data[i].LLI[0],obs->data[i].LLI[1]);
	}
}
static void dumpnav(nav_t *nav)
{
	int i;
	char str[64],s1[64],s2[64];
	printf("nav : n=%d\n",nav->n);
	for (i=0;i<nav->n;i++) {
		time2str(nav->eph[i].toe,str,3);
		time2str(nav->eph[i].toc,s1,0);
		time2str(nav->eph[i].ttr,s2,0);
		printf("%s : %2d    %s %s %3d %3d %2d\n",str,nav->eph[i].sat,s1,s2,
		       nav->eph[i].iode,nav->eph[i].iodc,nav->eph[i].svh);
	}
}
/* readrnx() */
void utest1(void)
{
	char *files1[]={"abc.00o","bcd.00n"};
	char *files2[]={"../07590920.05o","../07590920.05n","../30400920.05o","../30400920.05n"};
	obs_t obs;
	nav_t nav;
	int n;
	n=readrnx(files1,2,&obs,&nav);
		assert(n==0&&!obs.data&&!nav.eph);
	n=readrnx(files2,4,&obs,&nav);
		assert(n==2880&&obs.data&&obs.n>0&&nav.eph&&nav.n>0);
	dumpobs(&obs); dumpnav(&nav);
	free(obs.data);
	free(nav.eph);

	printf("%s utest1 : OK\n",__FILE__);
}
/* readrnxt() */
void utest2(void)
{
	gtime_t t0={0},ts,te;
	double ep1[]={2005,4,2,1,0,0},ep2[]={2005,4,2,2,0,0};
	char *files1[]={"../07590920.05o","../07590920.05n"};
	int n;
	obs_t obs;
	nav_t nav;
	ts=epoch2time(ep1);
	te=epoch2time(ep2);
	n=readrnxt(files1,2,ts,te,0.0,&obs,&nav);
	printf("\n\nn=%d\n",n);
	dumpobs(&obs);
	free(obs.data);
	n=readrnxt(files1,2,t0,t0,240.0,&obs,&nav);
	printf("\n\nn=%d\n",n);
	dumpobs(&obs);
	free(obs.data);

	printf("%s utset2 : OK\n",__FILE__);
}
static rnxopt_t opt1={{0}};
static rnxopt_t opt2= {
	"RROG567890123456789012345678901",
	"RUNBY67890123456789012345678901",
	"MARKER789012345678901234567890123456789012345678901234567890123",
	{"OBSERVER90123456789012345678901",
	 "AGENCY7890123456789012345678901"},
	{"RCV1567890123456789012345678901",
	 "RCV2567890123456789012345678901",
	 "RCV3567890123456789012345678901"},
	{"ANT1567890123456789012345678901",
	 "ANT2567890123456789012345678901",
	 "ANT3567890123456789012345678901"},
	{12345678.123,99999999.999,100000000.000},
	{123.0345,890123.9012,34567.0001},
	{"COMMENT1 012345678901234567890123456789012345678901234567890123",
	 "COMMENT2 012345678901234567890123456789012345678901234567890123",
	 "COMMENT3 012345678901234567890123456789012345678901234567890123",
	 "COMMENT4 012345678901234567890123456789012345678901234567890123",
	 "COMMENT5 012345678901234567890123456789012345678901234567890123"}
};
/* outrneobsh() */
void utest3(void)
{

	char tobs1[][3]={"P1","C1","P5","L1","L2","L5","S1","S2","S3","D1","D2","D5"};
	char tobs2[][3]={"AA"};
	outrnxobsh(stdout,&opt1,tobs1,12,timeget());
	outrnxobsh(stdout,&opt2,tobs2,1,timeget());
	
	printf("%s utest3 : OK\n",__FILE__);
}
/* outrneobsb() */
void utest4(void)
{
	char *files[]={"../07590920.05o"};
	char tobs1[][3]={"P1","C1","P5","L1","L2","L5","S1","S2","S3","D1","D2","D5"};
	char tobs2[][3]={"AA"};
	char tobs3[][3]={"L1","C1","L2","P2"};
	obs_t obs={0};
	int i,j;
	
	readrnx(files,1,&obs,NULL);
	outrnxobsb(stdout,obs.data,8,9,tobs1,12);
	outrnxobsb(stdout,obs.data,8,0,tobs2,1);
	
	for (i=j=0;i<obs.n;i=j) {
		while (j<obs.n&&timediff(obs.data[j].time,obs.data[i].time)<=0.0) j++;
		outrnxobsb(stdout,obs.data+i,j-i,0,tobs3,4);
	}
	printf("%s utest4 : OK\n",__FILE__);
}
/* outrnxnavh() */
void utest5(void)
{
	char *files[]={"../07590920.05n"};
	double ion[]={1E9,2E-4,3E8,4E3,-4E-3,-5E99,-6E-33,-9E-123};
	double utc[]={1E9,2E4,3E2,-9999};
	nav_t nav={0};
	readrnx(files,1,NULL,&nav);

	outrnxnavh(stdout,&opt1,ion,utc,15);
	outrnxnavh(stdout,&opt2,nav.ion,nav.utc,nav.leaps);
	
	printf("%s utest5 : OK\n",__FILE__);
}
/* outrnxnavb() */
void utest6(void)
{
	char *files[]={"../07590920.05n"};
	nav_t nav={0};
	int i;
	readrnx(files,1,NULL,&nav);
	for (i=0;i<nav.n;i++) {
		outrnxnavb(stdout,nav.eph+i);
	}
	printf("%s utest6 : OK\n",__FILE__);
}
int main(int argc, char **argv)
{
	utest1();
	utest2();
	utest3();
	utest4();
	utest5();
	utest6();
	return 0;
}
